@@ -13,16 +13,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from neutron_lib import constants as n_const
+from neutron_lib import context as nctx
+from neutron_lib.plugins.ml2 import api as driver_api
 
-from neutron import context as nctx
 import neutron.db.api as db
 from neutron.db import db_base_plugin_v2
 from neutron.db.models import segment as segment_models
+from neutron.db import models_v2
 from neutron.db import securitygroups_db as sec_db
 from neutron.db import segments_db
-from neutron.plugins.common import constants as p_const
-from neutron.plugins.ml2 import driver_api
 from neutron.plugins.ml2 import models as ml2_models
+from neutron.services.trunk import models as trunk_models
+from sqlalchemy import and_
 
 from networking_arista.common import db as db_models
 
@@ -34,7 +37,7 @@ def remember_tenant(tenant_id):
 
     :param tenant_id: globally unique neutron tenant identifier
     """
-    session = db.get_session()
+    session = db.get_writer_session()
     with session.begin():
         tenant = (session.query(db_models.AristaProvisionedTenants).
                   filter_by(tenant_id=tenant_id).first())
@@ -48,7 +51,7 @@ def forget_tenant(tenant_id):
 
     :param tenant_id: globally unique neutron tenant identifier
     """
-    session = db.get_session()
+    session = db.get_writer_session()
     with session.begin():
         (session.query(db_models.AristaProvisionedTenants).
          filter_by(tenant_id=tenant_id).
@@ -57,14 +60,14 @@ def forget_tenant(tenant_id):
 
 def get_all_tenants():
     """Returns a list of all tenants stored in repository."""
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         return session.query(db_models.AristaProvisionedTenants).all()
 
 
 def num_provisioned_tenants():
     """Returns number of tenants stored in repository."""
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         return session.query(db_models.AristaProvisionedTenants).count()
 
@@ -78,7 +81,7 @@ def remember_vm(vm_id, host_id, port_id, network_id, tenant_id):
     :param network_id: globally unique neutron network identifier
     :param tenant_id: globally unique neutron tenant identifier
     """
-    session = db.get_session()
+    session = db.get_writer_session()
     with session.begin():
         vm = db_models.AristaProvisionedVms(
             vm_id=vm_id,
@@ -94,7 +97,7 @@ def forget_all_ports_for_network(net_id):
 
     :param net_id: globally unique network ID
     """
-    session = db.get_session()
+    session = db.get_writer_session()
     with session.begin():
         (session.query(db_models.AristaProvisionedVms).
          filter_by(network_id=net_id).delete())
@@ -109,7 +112,7 @@ def update_port(vm_id, host_id, port_id, network_id, tenant_id):
     :param network_id: globally unique neutron network identifier
     :param tenant_id: globally unique neutron tenant identifier
     """
-    session = db.get_session()
+    session = db.get_writer_session()
     with session.begin():
         port = session.query(db_models.AristaProvisionedVms).filter_by(
             port_id=port_id).first()
@@ -127,7 +130,7 @@ def forget_port(port_id, host_id):
     :param port_id: globally unique port ID that connects VM to network
     :param host_id: host to which the port is bound to
     """
-    session = db.get_session()
+    session = db.get_writer_session()
     with session.begin():
         session.query(db_models.AristaProvisionedVms).filter_by(
             port_id=port_id,
@@ -143,7 +146,7 @@ def remember_network_segment(tenant_id,
     :param segmentation_id: segmentation id that is assigned to the network
     :param segment_id: globally unique neutron network segment identifier
     """
-    session = db.get_session()
+    session = db.get_writer_session()
     with session.begin():
         net = db_models.AristaProvisionedNets(
             tenant_id=tenant_id,
@@ -167,7 +170,7 @@ def forget_network_segment(tenant_id, network_id, segment_id=None):
     if segment_id:
         filters['id'] = segment_id
 
-    session = db.get_session()
+    session = db.get_writer_session()
     with session.begin():
         (session.query(db_models.AristaProvisionedNets).
          filter_by(**filters).delete())
@@ -179,7 +182,7 @@ def get_segmentation_id(tenant_id, network_id):
     :param tenant_id: globally unique neutron tenant identifier
     :param network_id: globally unique neutron network identifier
     """
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         net = (session.query(db_models.AristaProvisionedNets).
                filter_by(tenant_id=tenant_id,
@@ -198,7 +201,7 @@ def is_vm_provisioned(vm_id, host_id, port_id,
     :param network_id: globally unique neutron network identifier
     :param tenant_id: globally unique neutron tenant identifier
     """
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         num_vm = (session.query(db_models.AristaProvisionedVms).
                   filter_by(tenant_id=tenant_id,
@@ -223,7 +226,7 @@ def is_port_provisioned(port_id, host_id=None):
     if host_id:
         filters['host_id'] = host_id
 
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         num_ports = (session.query(db_models.AristaProvisionedVms).
                      filter_by(**filters).count())
@@ -239,7 +242,7 @@ def is_network_provisioned(tenant_id, network_id, segmentation_id=None,
     :param network_id: globally unique neutron network identifier
     :param segment_id: globally unique neutron network segment identifier
     """
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         filters = {'tenant_id': tenant_id,
                    'network_id': network_id}
@@ -260,7 +263,7 @@ def is_tenant_provisioned(tenant_id):
     :returns: True, if yes; False otherwise.
     :param tenant_id: globally unique neutron tenant identifier
     """
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         num_tenants = (session.query(db_models.AristaProvisionedTenants).
                        filter_by(tenant_id=tenant_id).count())
@@ -272,7 +275,7 @@ def num_nets_provisioned(tenant_id):
 
     :param tenant_id: globally unique neutron tenant identifier
     """
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         return (session.query(db_models.AristaProvisionedNets).
                 filter_by(tenant_id=tenant_id).count())
@@ -283,7 +286,7 @@ def num_vms_provisioned(tenant_id):
 
     :param tenant_id: globally unique neutron tenant identifier
     """
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         return (session.query(db_models.AristaProvisionedVms).
                 filter_by(tenant_id=tenant_id).count())
@@ -295,7 +298,7 @@ def get_networks(tenant_id):
     See AristaRPCWrapper.get_network_list() for return value format.
     :param tenant_id: globally unique neutron tenant identifier
     """
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         model = db_models.AristaProvisionedNets
         # hack for pep8 E711: comparison to None should be
@@ -323,7 +326,7 @@ def get_vms(tenant_id):
 
     :param tenant_id: globally unique neutron tenant identifier
     """
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         model = db_models.AristaProvisionedVms
         # hack for pep8 E711: comparison to None should be
@@ -363,7 +366,7 @@ def are_ports_attached_to_network(net_id):
 
     :param net_id: globally unique network ID
     """
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         model = db_models.AristaProvisionedVms
         return session.query(model).filter_by(network_id=net_id).filter(
@@ -375,7 +378,7 @@ def get_ports(tenant_id=None):
 
     :param tenant_id: globally unique neutron tenant identifier
     """
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         model = db_models.AristaProvisionedVms
         # hack for pep8 E711: comparison to None should be
@@ -406,7 +409,7 @@ def get_ports(tenant_id=None):
 
 def get_tenants():
     """Returns list of all tenants in EOS-compatible format."""
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         model = db_models.AristaProvisionedTenants
         all_tenants = session.query(model)
@@ -427,7 +430,7 @@ def _make_port_dict(record):
 
 def get_all_baremetal_ports():
     """Returns a list of all ports that belong to baremetal hosts."""
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         querry = session.query(ml2_models.PortBinding)
         bm_ports = querry.filter_by(vnic_type='baremetal').all()
@@ -438,7 +441,7 @@ def get_all_baremetal_ports():
 
 def get_all_portbindings():
     """Returns a list of all ports bindings."""
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         query = session.query(ml2_models.PortBinding)
         ports = query.all()
@@ -449,7 +452,7 @@ def get_all_portbindings():
 
 def get_port_binding_level(filters):
     """Returns entries from PortBindingLevel based on the specified filters."""
-    session = db.get_session()
+    session = db.get_reader_session()
     with session.begin():
         return (session.query(ml2_models.PortBindingLevel).
                 filter_by(**filters).
@@ -469,6 +472,45 @@ def get_network_segments_by_port_id(port_id):
         return [segment[0] for segment in segments]
 
 
+def get_trunk_port_by_subport_id(subport_id):
+    """Returns trunk parent port based on sub port id."""
+    session = db.get_reader_session()
+    with session.begin():
+        subport = (session.query(trunk_models.SubPort).
+                   filter_by(port_id=subport_id).first())
+        if subport:
+            trunk_id = subport.trunk_id
+            return get_trunk_port_by_trunk_id(trunk_id)
+
+
+def get_trunk_port_by_trunk_id(trunk_id):
+    session = db.get_reader_session()
+    with session.begin():
+        trunk_port = (session.query(trunk_models.Trunk).
+                      filter_by(id=trunk_id).first())
+        if trunk_port:
+            return trunk_port.port
+
+
+def get_unprovisioned_dvr_port_bindings():
+    """Returns DVR port bindings that don't exist in arista_provisioned_vms"""
+    session = db.get_reader_session()
+    # Hack for pep8 E711 to allow == none
+    none = None
+    with session.begin():
+        arista_vms_model = db_models.AristaProvisionedVms
+        distributed_pb_model = ml2_models.DistributedPortBinding
+        port_bindings = (session.query(distributed_pb_model, models_v2.Port).
+                         outerjoin(arista_vms_model,
+                                   and_(arista_vms_model.port_id ==
+                                        distributed_pb_model.port_id,
+                                        arista_vms_model.host_id ==
+                                        distributed_pb_model.host)).
+                         join(models_v2.Port).
+                         filter(arista_vms_model.port_id == none).all())
+        return port_bindings
+
+
 class NeutronNets(db_base_plugin_v2.NeutronDbPluginV2,
                   sec_db.SecurityGroupDbMixin):
     """Access to Neutron DB.
@@ -498,14 +540,14 @@ class NeutronNets(db_base_plugin_v2.NeutronDbPluginV2,
     def get_all_networks(self):
         return super(NeutronNets, self).get_networks(self.admin_ctx) or []
 
-    def get_all_ports(self):
-        return super(NeutronNets, self).get_ports(self.admin_ctx) or []
-
     def get_all_ports_for_tenant(self, tenant_id):
         filters = {'tenant_id': [tenant_id]}
         return super(NeutronNets,
                      self).get_ports(self.admin_ctx, filters=filters) or []
 
+    def get_all_ports(self):
+        return super(NeutronNets, self).get_ports(self.admin_ctx) or []
+
     def get_shared_network_owner_id(self, context, network_id):
         filters = {'id': [network_id]}
         nets = self.get_networks(context, filters=filters) or []
@@ -514,7 +556,7 @@ class NeutronNets(db_base_plugin_v2.NeutronDbPluginV2,
         if not nets or not segments:
             return
         if (nets[0]['shared'] and
-           segments[0][driver_api.NETWORK_TYPE] == p_const.TYPE_VLAN):
+           segments[0][driver_api.NETWORK_TYPE] == n_const.TYPE_VLAN):
             return nets[0]['tenant_id']
 
     def get_network_segments(self, network_id, dynamic=False, context=None):
@@ -605,3 +647,9 @@ class NeutronNets(db_base_plugin_v2.NeutronDbPluginV2,
         return super(NeutronNets,
                      self).get_security_group_rule(self.admin_ctx,
                                                    sec_gpr_id) or []
+
+    def validate_network_rbac_policy_change(self, resource, event, trigger,
+                                            context, object_type, policy,
+                                            **kwargs):
+        return super(NeutronNets, self).validate_network_rbac_policy_change(
+            resource, event, trigger, context, object_type, policy, kwargs)
