@@ -16,11 +16,11 @@ import hashlib
 import socket
 import struct
 
-import jsonrpc_requests
 from oslo_config import cfg
 from oslo_log import log as logging
 
 from networking_arista._i18n import _, _LI
+from networking_arista.common import api
 from networking_arista.common import exceptions as arista_exc
 
 LOG = logging.getLogger(__name__)
@@ -113,12 +113,7 @@ class AristaL3Driver(object):
         self._validate_config()
         host = cfg.CONF.l3_arista.primary_l3_host
         self._hosts.append(host)
-        self._servers.append(
-            jsonrpc_requests.Server(
-                self._eapi_host_url(host),
-                verify=False
-            )
-        )
+        self._servers.append(self._make_eapi_client(host))
         self._mlag_configured = cfg.CONF.l3_arista.mlag_config
         self._vrf_syntax_v2_supported = None
         self._router_in_vrf = router_in_vrf_v2
@@ -126,30 +121,27 @@ class AristaL3Driver(object):
         if self._mlag_configured:
             host = cfg.CONF.l3_arista.secondary_l3_host
             self._hosts.append(host)
-            self._servers.append(
-                jsonrpc_requests.Server(
-                    self._eapi_host_url(host),
-                    verify=False
-                )
-            )
+            self._servers.append(self._make_eapi_client(host))
             self._additionalRouterCmdsDict = additional_cmds_for_mlag['router']
             self._additionalInterfaceCmdsDict = (
                 additional_cmds_for_mlag['interface'])
         if self._use_vrf:
             self.routerDict = self._router_in_vrf['router']
-            self._update_vrf_commands()
+            self._update_vrf_commands(keep_alive=False)
             self._interfaceDict = self._router_in_vrf['interface']
         else:
             self.routerDict = router_in_default_vrf['router']
             self._interfaceDict = router_in_default_vrf['interface']
 
-    def _eapi_host_url(self, host):
-        user = cfg.CONF.l3_arista.primary_l3_host_username
-        pwd = cfg.CONF.l3_arista.primary_l3_host_password
-
-        eapi_server_url = ('https://%s:%s@%s/command-api' %
-                           (user, pwd, host))
-        return eapi_server_url
+    @staticmethod
+    def _make_eapi_client(host):
+        return api.EAPIClient(
+            host,
+            username=cfg.CONF.l3_arista.primary_l3_host_username,
+            password=cfg.CONF.l3_arista.primary_l3_host_password,
+            verify=False,
+            timeout=cfg.CONF.l3_arista.conn_timeout
+        )
 
     def _supports_vrf_instance(self, version):
         ver_tokens = version.split('.')
@@ -159,22 +151,26 @@ class AristaL3Driver(object):
             return False
         return True
 
-    def _check_vrf_syntax_v2_support(self, server):
+    def _check_vrf_syntax_v2_support(self, server, keep_alive=True):
         cmds = ['show version']
         result = None
         try:
-            result = server.runCmds(version=1, cmds=cmds)
+            result = server.execute(cmds, keep_alive=keep_alive)
+            LOG.info(_LI('Results of show version: %s'), result)
         except Exception:
-            return False
+            # We don't know what exception we got return None
+            # At this moment we don't know what command we support for vrf
+            # creation
+            return None
 
         return result and self._supports_vrf_instance(result[0].get('version',
                                                                     ''))
 
-    def _update_vrf_commands(self):
+    def _update_vrf_commands(self, keep_alive=True):
         # This assumes all switches run the same version. This needs to be
         # updated if we'll support distributed routing
         new_vrf_support = self._check_vrf_syntax_v2_support(
-            self._servers[0])
+            self._servers[0], keep_alive=keep_alive)
 
         if new_vrf_support == self._vrf_syntax_v2_supported:
             return
@@ -254,7 +250,7 @@ class AristaL3Driver(object):
             self._interfaceDict = self._router_in_vrf['interface']
         else:
             if ipv == 6:
-                # for IPv6 use IPv6 commands
+                # for IPv6 use IPv6 commmands
                 self._interfaceDict = router_in_default_vrf_v6['interface']
                 self._additionalInterfaceCmdsDict = (
                     additional_cmds_for_mlag_v6['interface'])
@@ -428,7 +424,8 @@ class AristaL3Driver(object):
                         LOG.exception(msg)
                         raise arista_exc.AristaServicePluginRpcError(msg=msg)
 
-    def _run_openstack_l3_cmds(self, commands, server, log_exception=True):
+    def _run_openstack_l3_cmds(self, commands, server, log_exception=True,
+                               keep_alive=True):
         """Execute/sends a CAPI (Command API) command to EOS.
 
         In this method, list of commands is appended with prefix and
@@ -446,13 +443,12 @@ class AristaL3Driver(object):
         try:
             # this returns array of return values for every command in
             # full_command list
-            ret = server.runCmds(version=1, cmds=full_command)
+            ret = server.execute(full_command, keep_alive=keep_alive)
             LOG.info(_LI('Results of execution on Arista EOS: %s'), ret)
 
-        except Exception:
-            msg = (_('Error occurred while trying to execute '
-                     'commands %(cmd)s on EOS %(host)s') %
-                   {'cmd': full_command, 'host': server})
+        except arista_exc.AristaServicePluginInvalidCommand:
+            msg = (_('VRF creation command unsupported. This request should '
+                     'work on next retry.'))
             if log_exception:
                 LOG.exception(msg)
             if self._use_vrf:
@@ -460,6 +456,13 @@ class AristaL3Driver(object):
                 # exception is vrf instance/definition and we need to update
                 # the current support
                 self._update_vrf_commands()
+            raise
+        except Exception:
+            msg = (_('Error occurred while trying to execute '
+                     'commands %(cmd)s on EOS %(host)s') %
+                   {'cmd': full_command, 'host': server})
+            if log_exception:
+                LOG.exception(msg)
             raise arista_exc.AristaServicePluginRpcError(msg=msg)
 
     def _arista_router_name(self, tenant_id, name):
