@@ -13,42 +13,37 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import json
 import threading
 
 from neutron_lib.api.definitions import portbindings
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
 from neutron_lib import constants as n_const
+from neutron_lib.plugins.ml2 import api as driver_api
 from oslo_concurrency import lockutils
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import excutils
 
-from neutron.common import constants as neutron_const
-from neutron.plugins.common import constants as p_const
-from neutron.plugins.ml2.common import exceptions as ml2_exc
-from neutron.plugins.ml2 import driver_api
+from neutron.services.trunk import constants as trunk_consts
 
 from networking_arista._i18n import _, _LI, _LE
+from networking_arista.common import constants
 from networking_arista.common import db
 from networking_arista.common import db_lib
 from networking_arista.common import exceptions as arista_exc
-from networking_arista.ml2 import arista_ml2
+from networking_arista.ml2 import arista_sync
+from networking_arista.ml2.rpc.arista_eapi import AristaRPCWrapperEapi
+from networking_arista.ml2.rpc.arista_json import AristaRPCWrapperJSON
 from networking_arista.ml2 import sec_group_callback
 
+
 LOG = logging.getLogger(__name__)
 cfg.CONF.import_group('ml2_arista', 'networking_arista.common.config')
 
-# Messages
-EOS_UNREACHABLE_MSG = _('Unable to reach EOS')
-UNABLE_TO_DELETE_PORT_MSG = _('Unable to delete port from EOS')
-UNABLE_TO_DELETE_DEVICE_MSG = _('Unable to delete device')
-
-# Constants
-INTERNAL_TENANT_ID = 'INTERNAL-TENANT-ID'
-PORT_BINDING_HOST = 'binding:host_id'
-MECHANISM_DRV_NAME = 'arista'
-
 
 def pretty_log(tag, obj):
-    import json
     log_data = json.dumps(obj, sort_keys=True, indent=4)
     LOG.debug(tag)
     LOG.debug(log_data)
@@ -71,7 +66,6 @@ class AristaDriver(driver_api.MechanismDriver):
         confg = cfg.CONF.ml2_arista
         self.segmentation_type = db_lib.VLAN_SEGMENTATION
         self.timer = None
-        self.sync_timeout = confg['sync_interval']
         self.managed_physnets = confg['managed_physnets']
         self.manage_fabric = confg['manage_fabric']
         self.eos_sync_lock = threading.Lock()
@@ -83,33 +77,32 @@ class AristaDriver(driver_api.MechanismDriver):
             self.rpc = rpc
             self.eapi = rpc
         else:
-            self.eapi = arista_ml2.AristaRPCWrapperEapi(self.ndb)
+            self.eapi = AristaRPCWrapperEapi(self.ndb)
             api_type = confg['api_type'].upper()
             if api_type == 'EAPI':
                 LOG.info("Using EAPI for RPC")
-                self.rpc = arista_ml2.AristaRPCWrapperEapi(self.ndb)
+                self.rpc = AristaRPCWrapperEapi(self.ndb)
             elif api_type == 'JSON':
                 LOG.info("Using JSON for RPC")
-                self.rpc = arista_ml2.AristaRPCWrapperJSON(self.ndb)
+                self.rpc = AristaRPCWrapperJSON(self.ndb)
             else:
                 msg = "RPC mechanism %s not recognized" % api_type
                 LOG.error(msg)
                 raise arista_exc.AristaRpcError(msg=msg)
 
-        self.sync_service = arista_ml2.SyncService(self.rpc, self.ndb)
-        self.rpc.sync_service = self.sync_service
-
     def initialize(self):
         if self.rpc.check_cvx_availability():
             self.rpc.register_with_eos()
             self.rpc.check_supported_features()
 
-        self._cleanup_db()
-        # Registering with EOS updates self.rpc.region_updated_time. Clear it
-        # to force an initial sync
-        self.rpc.clear_region_updated_time()
-        self._synchronization_thread()
         self.sg_handler = sec_group_callback.AristaSecurityGroupHandler(self)
+        registry.subscribe(self.set_subport,
+                           trunk_consts.SUBPORTS, events.AFTER_CREATE)
+        registry.subscribe(self.unset_subport,
+                           trunk_consts.SUBPORTS, events.AFTER_DELETE)
+
+    def get_workers(self):
+        return [arista_sync.AristaSyncWorker(self.rpc, self.ndb)]
 
     def create_network_precommit(self, context):
         """Remember the tenant, and network information."""
@@ -120,10 +113,10 @@ class AristaDriver(driver_api.MechanismDriver):
             # Hierarchical port binding is not supported by CVX, only
             # allow VLAN network type.
             if(segments and
-                    segments[0][driver_api.NETWORK_TYPE] != p_const.TYPE_VLAN):
+                    segments[0][driver_api.NETWORK_TYPE] != n_const.TYPE_VLAN):
                 return
         network_id = network['id']
-        tenant_id = network['tenant_id'] or INTERNAL_TENANT_ID
+        tenant_id = network['tenant_id'] or constants.INTERNAL_TENANT_ID
         with self.eos_sync_lock:
             db_lib.remember_tenant(tenant_id)
             for segment in segments:
@@ -138,7 +131,7 @@ class AristaDriver(driver_api.MechanismDriver):
         network = context.current
         network_id = network['id']
         network_name = network['name']
-        tenant_id = network['tenant_id'] or INTERNAL_TENANT_ID
+        tenant_id = network['tenant_id'] or constants.INTERNAL_TENANT_ID
         segments = context.network_segments
         shared_net = network['shared']
         with self.eos_sync_lock:
@@ -182,7 +175,8 @@ class AristaDriver(driver_api.MechanismDriver):
            (new_network['shared'] != orig_network['shared'])):
             network_id = new_network['id']
             network_name = new_network['name']
-            tenant_id = new_network['tenant_id'] or INTERNAL_TENANT_ID
+            tenant_id = (new_network['tenant_id'] or
+                         constants.INTERNAL_TENANT_ID)
             shared_net = new_network['shared']
             with self.eos_sync_lock:
                 if db_lib.is_network_provisioned(tenant_id, network_id):
@@ -206,7 +200,7 @@ class AristaDriver(driver_api.MechanismDriver):
         """Delete the network information from the DB."""
         network = context.current
         network_id = network['id']
-        tenant_id = network['tenant_id'] or INTERNAL_TENANT_ID
+        tenant_id = network['tenant_id'] or constants.INTERNAL_TENANT_ID
         with self.eos_sync_lock:
             if db_lib.is_network_provisioned(tenant_id, network_id):
                 if db_lib.are_ports_attached_to_network(network_id):
@@ -223,14 +217,14 @@ class AristaDriver(driver_api.MechanismDriver):
             # Hierarchical port binding is not supported by CVX, only
             # send the request if network type is VLAN.
             if (segments and
-                    segments[0][driver_api.NETWORK_TYPE] != p_const.TYPE_VLAN):
+                    segments[0][driver_api.NETWORK_TYPE] != n_const.TYPE_VLAN):
                 # If network type is not VLAN, do nothing
                 return
             # No need to pass segments info when calling delete_network as
             # HPB is not supported.
             segments = []
         network_id = network['id']
-        tenant_id = network['tenant_id'] or INTERNAL_TENANT_ID
+        tenant_id = network['tenant_id'] or constants.INTERNAL_TENANT_ID
         with self.eos_sync_lock:
 
             # Succeed deleting network in case EOS is not accessible.
@@ -298,7 +292,7 @@ class AristaDriver(driver_api.MechanismDriver):
         context.set_binding(segment[driver_api.ID],
                             portbindings.VIF_TYPE_OTHER,
                             vif_details,
-                            p_const.ACTIVE)
+                            n_const.ACTIVE)
         LOG.debug("AristaDriver: bound port info- port ID %(id)s "
                   "on network %(network)s",
                   {'id': port['id'],
@@ -340,7 +334,7 @@ class AristaDriver(driver_api.MechanismDriver):
                 LOG.debug("bind_port for port %(port)s: no physical_network "
                           "found", {'port': port.get('id')})
                 continue
-            if segment[driver_api.NETWORK_TYPE] == p_const.TYPE_VXLAN:
+            if segment[driver_api.NETWORK_TYPE] == n_const.TYPE_VXLAN:
                 # Check if CVX supports HPB
                 if not self.rpc.hpb_supported():
                     LOG.debug("bind_port: HPB is not supported")
@@ -394,7 +388,8 @@ class AristaDriver(driver_api.MechanismDriver):
         if any([device_owner in supported_device_owner,
                 device_owner.startswith('compute') and
                 device_owner != 'compute:probe',
-                device_owner.startswith('baremetal')]):
+                device_owner.startswith('baremetal'),
+                device_owner.startswith('trunk')]):
             return True
 
         LOG.debug('Unsupported device owner: %s', device_owner)
@@ -487,7 +482,7 @@ class AristaDriver(driver_api.MechanismDriver):
             LOG.debug("Handling port %s for: %s " % (
                 "migration" if is_port_migrated else "delete", orig_port))
             network_id = orig_port['network_id']
-            tenant_id = orig_port['tenant_id'] or INTERNAL_TENANT_ID
+            tenant_id = orig_port['tenant_id'] or constants.INTERNAL_TENANT_ID
             # Ensure that we use tenant Id for the network owner
             tenant_id = self._network_owner_tenant(context, network_id,
                                                    tenant_id)
@@ -526,7 +521,7 @@ class AristaDriver(driver_api.MechanismDriver):
             # 2. If segment_id is provisioned and it not bound to any port it
             # should be removed from EOS.
             network_id = orig_port['network_id']
-            tenant_id = orig_port['tenant_id'] or INTERNAL_TENANT_ID
+            tenant_id = orig_port['tenant_id'] or constants.INTERNAL_TENANT_ID
             # Ensure that we use tenant Id for the network owner
             tenant_id = self._network_owner_tenant(context, network_id,
                                                    tenant_id)
@@ -562,7 +557,7 @@ class AristaDriver(driver_api.MechanismDriver):
                                     tenant_id, network_id,
                                     binding_level.segment_id)
                             except arista_exc.AristaRpcError:
-                                LOG.info(EOS_UNREACHABLE_MSG)
+                                LOG.info(constants.EOS_UNREACHABLE_MSG)
 
             return True
 
@@ -603,7 +598,7 @@ class AristaDriver(driver_api.MechanismDriver):
         # device_id and device_owner are set on VM boot
         port_id = new_port['id']
         network_id = new_port['network_id']
-        tenant_id = new_port['tenant_id'] or INTERNAL_TENANT_ID
+        tenant_id = new_port['tenant_id'] or constants.INTERNAL_TENANT_ID
         # Ensure that we use tenant Id for the network owner
         tenant_id = self._network_owner_tenant(context, network_id, tenant_id)
 
@@ -685,6 +680,11 @@ class AristaDriver(driver_api.MechanismDriver):
         host = context.host
         is_vm_boot = device_id and device_owner
 
+        # When delete a vm, the trunk port context has no device_owner
+        # Keep device_owner as in original port
+        if not device_owner and orig_port.get('trunk_details'):
+            device_owner = orig_port['device_owner']
+
         if not self._supported_device_owner(device_owner):
             return
 
@@ -697,7 +697,7 @@ class AristaDriver(driver_api.MechanismDriver):
         port_id = port['id']
         port_name = port['name']
         network_id = port['network_id']
-        tenant_id = port['tenant_id'] or INTERNAL_TENANT_ID
+        tenant_id = port['tenant_id'] or constants.INTERNAL_TENANT_ID
         # Ensure that we use tenant Id for the network owner
         tenant_id = self._network_owner_tenant(context, network_id, tenant_id)
         sg = port['security_groups']
@@ -711,11 +711,14 @@ class AristaDriver(driver_api.MechanismDriver):
             # Return from here as port migration is already handled.
             return
 
-        seg_info = self._bound_segments(context)
-        if not seg_info:
-            LOG.debug("Ignoring the update as the port is not managed by "
-                      "Arista switches.")
-            return
+        # Check if it is trunk_port deletion case
+        seg_info = []
+        if not port.get('trunk_details') or host:
+            seg_info = self._bound_segments(context)
+            if not seg_info:
+                LOG.debug("Ignoring the update as the port is not managed by "
+                          "Arista switches.")
+                return
 
         with self.eos_sync_lock:
             hostname = self._host_name(host)
@@ -752,40 +755,33 @@ class AristaDriver(driver_api.MechanismDriver):
                         tenant_id, network_id,
                         context.network.current['name'], all_segments)
                 except arista_exc.AristaRpcError:
-                    LOG.error(_LE("Failed to create network segments"))
-                    raise ml2_exc.MechanismDriverError(
-                        method='update_port_postcommit')
+                    with excutils.save_and_reraise_exception():
+                        LOG.error(_LE("Failed to create network segments"))
 
             try:
                 orig_host = context.original_host
                 port_down = False
-                if(port['device_owner'] == n_const.DEVICE_OWNER_DVR_INTERFACE):
-                    # We care about port status only for DVR ports
+                if(port['device_owner'] == n_const.DEVICE_OWNER_DVR_INTERFACE
+                   or port.get('trunk_details')):
+                    # We care about port status only for DVR ports and
+                    # trunk ports
                     port_down = context.status == n_const.PORT_STATUS_DOWN
 
                 if orig_host and (port_down or host != orig_host or
-                   device_id == neutron_const.DEVICE_ID_RESERVED_DHCP_PORT):
-                    try:
-                        LOG.info("Deleting the port %s" % str(orig_port))
-                        # The port moved to a different host or the VM
-                        # connected to the port was deleted or its in DOWN
-                        # state. So delete the old port on the old host.
-                        self._delete_port(
-                            context,
-                            orig_port,
-                            orig_host,
-                            tenant_id
-                        )
-                    except ml2_exc.MechanismDriverError:
-                        # If deleting a port fails, then not much can be done
-                        # about it. Log a warning and move on.
-                        LOG.warning(UNABLE_TO_DELETE_PORT_MSG)
+                   device_id == n_const.DEVICE_ID_RESERVED_DHCP_PORT):
+                    LOG.info("Deleting the port %s" % str(orig_port))
+                    # The port moved to a different host or the VM
+                    # connected to the port was deleted or its in DOWN
+                    # state. So delete the old port on the old host.
+                    self._delete_port(context, orig_port, orig_host, tenant_id)
+
                 if(port_provisioned and net_provisioned and hostname and
                    is_vm_boot and not port_down and
-                   device_id != neutron_const.DEVICE_ID_RESERVED_DHCP_PORT):
+                   device_id != n_const.DEVICE_ID_RESERVED_DHCP_PORT):
                     LOG.info(_LI("Port plugged into network"))
                     # Plug port into the network only if it exists in the db
                     # and is bound to a host and the port is up.
+                    trunk_details = port.get('trunk_details')
                     self.rpc.plug_port_into_network(device_id,
                                                     hostname,
                                                     port_id,
@@ -796,7 +792,9 @@ class AristaDriver(driver_api.MechanismDriver):
                                                     sg, orig_sg,
                                                     vnic_type,
                                                     segments=segments,
-                                                    switch_bindings=bindings)
+                                                    switch_bindings=bindings,
+                                                    trunk_details=trunk_details
+                                                    )
                 else:
                     LOG.info(_LI("Port not plugged into network"))
             except arista_exc.AristaRpcError as err:
@@ -826,7 +824,7 @@ class AristaDriver(driver_api.MechanismDriver):
         host = context.host
         network_id = port['network_id']
 
-        tenant_id = port['tenant_id'] or INTERNAL_TENANT_ID
+        tenant_id = port['tenant_id'] or constants.INTERNAL_TENANT_ID
         # Ensure that we use tenant Id for the network owner
         tenant_id = self._network_owner_tenant(context, network_id, tenant_id)
 
@@ -841,10 +839,10 @@ class AristaDriver(driver_api.MechanismDriver):
             try:
                 self._delete_port(context, port, host, tenant_id)
                 self._delete_segment(context, tenant_id)
-            except ml2_exc.MechanismDriverError:
+            except arista_exc.AristaRpcError:
                 # Can't do much if deleting a port failed.
                 # Log a warning and continue.
-                LOG.warning(UNABLE_TO_DELETE_PORT_MSG)
+                LOG.warning(constants.UNABLE_TO_DELETE_PORT_MSG)
 
     def _delete_port(self, context, port, host, tenant_id):
         """Deletes the port from EOS.
@@ -871,21 +869,23 @@ class AristaDriver(driver_api.MechanismDriver):
         sg = port['security_groups']
 
         if not device_id or not host:
-            LOG.warning(UNABLE_TO_DELETE_DEVICE_MSG)
+            LOG.warning(constants.UNABLE_TO_DELETE_DEVICE_MSG)
             return
 
         try:
             hostname = self._host_name(host)
+            trunk_details = port.get('trunk_details')
             self.rpc.unplug_port_from_network(device_id, device_owner,
                                               hostname, port_id, network_id,
                                               tenant_id, sg, vnic_type,
-                                              switch_bindings=switch_bindings)
+                                              switch_bindings=switch_bindings,
+                                              trunk_details=trunk_details)
             self.rpc.remove_security_group(sg, switch_bindings)
 
             # if necessary, delete tenant as well.
             self.delete_tenant(tenant_id)
         except arista_exc.AristaRpcError:
-            LOG.info(EOS_UNREACHABLE_MSG)
+            LOG.info(constants.EOS_UNREACHABLE_MSG)
 
     def _delete_segment(self, context, tenant_id):
         """Deletes a dynamic network segment from EOS.
@@ -923,7 +923,7 @@ class AristaDriver(driver_api.MechanismDriver):
                         db_lib.forget_network_segment(
                             tenant_id, network_id, binding_level.segment_id)
                     except arista_exc.AristaRpcError:
-                        LOG.info(EOS_UNREACHABLE_MSG)
+                        LOG.info(constants.EOS_UNREACHABLE_MSG)
                 else:
                     LOG.debug("Cannot delete segment_id %(segid)s "
                               "segment is %(seg)s",
@@ -952,7 +952,7 @@ class AristaDriver(driver_api.MechanismDriver):
             driver = binding_level.get(driver_api.BOUND_DRIVER)
             bound_drivers.append(driver)
             if (bound_segment and
-                    bound_segment.get('network_type') == p_const.TYPE_VLAN):
+                    bound_segment.get('network_type') == n_const.TYPE_VLAN):
                 segment_id = bound_segment.get('id')
                 break
 
@@ -961,7 +961,8 @@ class AristaDriver(driver_api.MechanismDriver):
         # When Arista driver participate in port binding by allocating dynamic
         # segment and then calling continue_binding, the driver should the
         # second last driver in the bound drivers list.
-        if (segment_id and bound_drivers[-2:-1] == [MECHANISM_DRV_NAME]):
+        if (segment_id and bound_drivers[-2:-1] ==
+                [constants.MECHANISM_DRV_NAME]):
             filters = {'segment_id': segment_id}
             result = db_lib.get_port_binding_level(filters)
             LOG.debug("Looking for entry with filters=%(filters)s "
@@ -988,44 +989,13 @@ class AristaDriver(driver_api.MechanismDriver):
             try:
                 self.rpc.delete_tenant(tenant_id)
             except arista_exc.AristaRpcError:
-                LOG.info(EOS_UNREACHABLE_MSG)
-                raise ml2_exc.MechanismDriverError(method='delete_tenant')
+                with excutils.save_and_reraise_exception():
+                    LOG.info(constants.EOS_UNREACHABLE_MSG)
 
     def _host_name(self, hostname):
         fqdns_used = cfg.CONF.ml2_arista['use_fqdn']
         return hostname if fqdns_used else hostname.split('.')[0]
 
-    def _synchronization_thread(self):
-        with self.eos_sync_lock:
-            try:
-                self.sync_service.do_synchronize()
-            except Exception as e:
-                LOG.warning(e)
-
-        self.timer = threading.Timer(self.sync_timeout,
-                                     self._synchronization_thread)
-        self.timer.start()
-
-    def stop_synchronization_thread(self):
-        if self.timer:
-            self.timer.cancel()
-            self.timer = None
-
-    def _cleanup_db(self):
-        """Clean up any unnecessary entries in our DB."""
-        neutron_nets = self.ndb.get_all_networks()
-        arista_db_nets = db_lib.get_networks(tenant_id='any')
-        neutron_net_ids = set()
-        for net in neutron_nets:
-            neutron_net_ids.add(net['id'])
-
-        # Remove networks from the Arista DB if the network does not exist in
-        # Neutron DB
-        for net_id in set(arista_db_nets.keys()).difference(neutron_net_ids):
-            tenant_network = arista_db_nets[net_id]
-            db_lib.forget_network_segment(tenant_network['tenantId'], net_id)
-            db_lib.forget_all_ports_for_network(net_id)
-
     def _network_provisioned(self, context, tenant_id, network_id,
                              segmentation_id=None, segment_id=None):
         # If network does not exist under this tenant,
@@ -1082,3 +1052,80 @@ class AristaDriver(driver_api.MechanismDriver):
                     msg = (_('Failed to delete ACL rule on EOS %s') % sgr)
                     LOG.exception(msg)
                     raise arista_exc.AristaSecurityGroupError(msg=msg)
+
+    def unset_subport(self, resource, event, trigger, **kwargs):
+        payload = kwargs['payload']
+        trunk_id = payload.trunk_id
+        subports = payload.subports
+
+        trunk_port = db_lib.get_trunk_port_by_trunk_id(trunk_id)
+        if trunk_port:
+            device_id = trunk_port.device_id
+            tenant_id = trunk_port.tenant_id
+            host = trunk_port.port_binding.host
+            vnic_type = trunk_port.port_binding.vnic_type
+            profile = trunk_port.port_binding.profile
+            if profile:
+                profile = json.loads(profile)
+
+            for subport in subports:
+                subport_id = subport.port_id
+                subport_current = self.ndb.get_port(subport_id)
+                subport_current['device_id'] = device_id
+                subport_current['binding:vnic_type'] = vnic_type
+                subport_current['binding:profile'] = profile
+                subport_current['device_owner'] = 'trunk:subport'
+
+                with self.eos_sync_lock:
+                    self._delete_port(
+                        payload.context,
+                        subport_current,
+                        host,
+                        tenant_id
+                    )
+        else:
+            LOG.warning('Unable to unset the subport, no trunk port found')
+
+    def set_subport(self, resource, event, trigger, **kwargs):
+        payload = kwargs['payload']
+        trunk_id = payload.trunk_id
+        subports = payload.subports
+
+        device_owner = 'trunk:subport'
+        trunk_port = db_lib.get_trunk_port_by_trunk_id(trunk_id)
+        if not trunk_port:
+            return
+        device_id = trunk_port.device_id
+        tenant_id = trunk_port.tenant_id
+        host = trunk_port.port_binding.host
+        if not host:
+            return
+        hostname = self._host_name(host)
+        vnic_type = trunk_port.port_binding.vnic_type
+
+        profile = trunk_port.port_binding.profile
+        bindings = []
+        if profile:
+            profile = json.loads(profile)
+            bindings = profile.get('local_link_information', [])
+
+        for subport in subports:
+            subport_id = subport.port_id
+            subport_current = self.ndb.get_port(subport_id)
+            network_id = self.ndb.get_network_id_from_port_id(subport_id)
+            port_name = subport_current.get('name')
+            sg = subport_current.get('security_groups')
+            orig_sg = None
+            segments = db_lib.get_network_segments_by_port_id(subport_id)
+
+            self.rpc.plug_port_into_network(device_id,
+                                            hostname,
+                                            subport_id,
+                                            network_id,
+                                            tenant_id,
+                                            port_name,
+                                            device_owner,
+                                            sg, orig_sg,
+                                            vnic_type,
+                                            segments=segments,
+                                            switch_bindings=bindings)
