@@ -57,8 +57,8 @@ class AristaL3DriverTestCasesDefaultVrf(base.BaseTestCase):
                                       self.drv._servers[0])
         cmds = ['enable', 'configure', 'exit']
 
-        self.drv._servers[0].runCmds.assert_called_once_with(version=1,
-                                                             cmds=cmds)
+        self.drv._servers[0].execute.assert_called_once_with(cmds,
+                                                             keep_alive=True)
 
     def test_delete_router_from_eos(self):
         router_name = 'test-router-1'
@@ -66,8 +66,8 @@ class AristaL3DriverTestCasesDefaultVrf(base.BaseTestCase):
         self.drv.delete_router_from_eos(router_name, self.drv._servers[0])
         cmds = ['enable', 'configure', 'exit']
 
-        self.drv._servers[0].runCmds.assert_called_once_with(version=1,
-                                                             cmds=cmds)
+        self.drv._servers[0].execute.assert_called_once_with(cmds,
+                                                             keep_alive=True)
 
     def test_add_interface_to_router_on_eos(self):
         router_name = 'test-router-1'
@@ -83,8 +83,8 @@ class AristaL3DriverTestCasesDefaultVrf(base.BaseTestCase):
                 'interface vlan %s' % segment_id,
                 'ip address %s/%s' % (gw_ip, mask), 'exit']
 
-        self.drv._servers[0].runCmds.assert_called_once_with(version=1,
-                                                             cmds=cmds)
+        self.drv._servers[0].execute.assert_called_once_with(cmds,
+                                                             keep_alive=True)
 
     def test_delete_interface_from_router_on_eos(self):
         router_name = 'test-router-1'
@@ -95,8 +95,8 @@ class AristaL3DriverTestCasesDefaultVrf(base.BaseTestCase):
         cmds = ['enable', 'configure', 'no interface vlan %s' % segment_id,
                 'exit']
 
-        self.drv._servers[0].runCmds.assert_called_once_with(version=1,
-                                                             cmds=cmds)
+        self.drv._servers[0].execute.assert_called_once_with(cmds,
+                                                             keep_alive=True)
 
 
 class AristaL3DriverTestCasesUsingVRFs(base.BaseTestCase):
@@ -125,7 +125,7 @@ class AristaL3DriverTestCasesUsingVRFs(base.BaseTestCase):
         with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
             c.return_value = True
             self.drv._update_vrf_commands()
-            c.assert_called_once_with(self.drv._servers[0])
+            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)
 
         for (r, d) in zip(routers, domains):
             self.drv.create_router_on_eos(r, d, self.drv._servers[0])
@@ -134,8 +134,8 @@ class AristaL3DriverTestCasesUsingVRFs(base.BaseTestCase):
                     'vrf instance %s' % r,
                     'rd %(rd)s:%(rd)s' % {'rd': d}, 'exit', 'exit']
 
-            self.drv._servers[0].runCmds.assert_called_with(cmds=cmds,
-                                                            version=1)
+            self.drv._servers[0].execute.assert_called_with(cmds,
+                                                            keep_alive=True)
 
     def test_create_router_on_eos_v1_syntax(self):
         max_vrfs = 5
@@ -144,7 +144,7 @@ class AristaL3DriverTestCasesUsingVRFs(base.BaseTestCase):
         with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
             c.return_value = False
             self.drv._update_vrf_commands()
-            c.assert_called_once_with(self.drv._servers[0])
+            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)
 
         for (r, d) in zip(routers, domains):
             self.drv.create_router_on_eos(r, d, self.drv._servers[0])
@@ -153,8 +153,8 @@ class AristaL3DriverTestCasesUsingVRFs(base.BaseTestCase):
                     'vrf definition %s' % r,
                     'rd %(rd)s:%(rd)s' % {'rd': d}, 'exit', 'exit']
 
-            self.drv._servers[0].runCmds.assert_called_with(version=1,
-                                                            cmds=cmds)
+            self.drv._servers[0].execute.assert_called_with(cmds,
+                                                            keep_alive=True)
 
     def test_delete_router_from_eos_v1_syntax(self):
         max_vrfs = 5
@@ -163,14 +163,14 @@ class AristaL3DriverTestCasesUsingVRFs(base.BaseTestCase):
         with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
             c.return_value = False
             self.drv._update_vrf_commands()
-            c.assert_called_once_with(self.drv._servers[0])
+            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)
 
         for r in routers:
             self.drv.delete_router_from_eos(r, self.drv._servers[0])
             cmds = ['enable', 'configure', 'no vrf definition %s' % r, 'exit']
 
-            self.drv._servers[0].runCmds.assert_called_with(cmds=cmds,
-                                                            version=1)
+            self.drv._servers[0].execute.assert_called_with(cmds,
+                                                            keep_alive=True)
 
     def test_delete_router_from_eos_v2_syntax(self):
         max_vrfs = 5
@@ -179,14 +179,14 @@ class AristaL3DriverTestCasesUsingVRFs(base.BaseTestCase):
         with mock.patch.object(self.drv, '_check_vrf_syntax_v2_support') as c:
             c.return_value = True
             self.drv._update_vrf_commands()
-            c.assert_called_once_with(self.drv._servers[0])
+            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)
 
         for r in routers:
             self.drv.delete_router_from_eos(r, self.drv._servers[0])
             cmds = ['enable', 'configure', 'no vrf instance %s' % r, 'exit']
 
-            self.drv._servers[0].runCmds.assert_called_with(version=1,
-                                                            cmds=cmds)
+            self.drv._servers[0].execute.assert_called_with(cmds,
+                                                            keep_alive=True)
 
     def test_add_interface_to_router_on_eos_v1(self):
         router_name = 'test-router-1'
@@ -199,7 +199,7 @@ class AristaL3DriverTestCasesUsingVRFs(base.BaseTestCase):
             c.return_value = False
             self.drv._update_vrf_commands()
             self.drv._select_dicts(4)
-            c.assert_called_once_with(self.drv._servers[0])
+            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)
 
         self.drv.add_interface_to_router(segment_id, router_name, gw_ip,
                                          router_ip, mask, self.drv._servers[0])
@@ -210,8 +210,8 @@ class AristaL3DriverTestCasesUsingVRFs(base.BaseTestCase):
                 'vrf forwarding %s' % router_name,
                 'ip address %s/%s' % (gw_ip, mask), 'exit']
 
-        self.drv._servers[0].runCmds.assert_called_with(version=1,
-                                                        cmds=cmds)
+        self.drv._servers[0].execute.assert_called_once_with(cmds,
+                                                             keep_alive=True)
 
     def test_add_interface_to_router_on_eos_v2(self):
         router_name = 'test-router-1'
@@ -224,7 +224,7 @@ class AristaL3DriverTestCasesUsingVRFs(base.BaseTestCase):
             c.return_value = True
             self.drv._update_vrf_commands()
             self.drv._select_dicts(4)
-            c.assert_called_once_with(self.drv._servers[0])
+            c.assert_called_once_with(self.drv._servers[0], keep_alive=True)
 
         self.drv.add_interface_to_router(segment_id, router_name, gw_ip,
                                          router_ip, mask, self.drv._servers[0])
@@ -235,8 +235,8 @@ class AristaL3DriverTestCasesUsingVRFs(base.BaseTestCase):
                 'vrf %s' % router_name,
                 'ip address %s/%s' % (gw_ip, mask), 'exit']
 
-        self.drv._servers[0].runCmds.assert_called_with(version=1,
-                                                        cmds=cmds)
+        self.drv._servers[0].execute.assert_called_once_with(cmds,
+                                                             keep_alive=True)
 
     def test_delete_interface_from_router_on_eos(self):
         router_name = 'test-router-1'
@@ -247,8 +247,8 @@ class AristaL3DriverTestCasesUsingVRFs(base.BaseTestCase):
         cmds = ['enable', 'configure', 'no interface vlan %s' % segment_id,
                 'exit']
 
-        self.drv._servers[0].runCmds.assert_called_once_with(version=1,
-                                                             cmds=cmds)
+        self.drv._servers[0].execute.assert_called_once_with(cmds,
+                                                             keep_alive=True)
 
 
 class AristaL3DriverTestCasesMlagConfig(base.BaseTestCase):
@@ -281,7 +281,7 @@ class AristaL3DriverTestCasesMlagConfig(base.BaseTestCase):
             cmds = ['enable', 'configure',
                     'ip virtual-router mac-address %s' % router_mac, 'exit']
 
-            s.runCmds.assert_called_with(version=1, cmds=cmds)
+            s.execute.assert_called_with(cmds, keep_alive=True)
 
     def test_delete_router_from_eos(self):
         router_name = 'test-router-1'
@@ -290,7 +290,7 @@ class AristaL3DriverTestCasesMlagConfig(base.BaseTestCase):
             self.drv.delete_router_from_eos(router_name, s)
             cmds = ['enable', 'configure', 'exit']
 
-            s.runCmds.assert_called_once_with(version=1, cmds=cmds)
+            s.execute.assert_called_once_with(cmds, keep_alive=True)
 
     def test_add_interface_to_router_on_eos(self):
         router_name = 'test-router-1'
@@ -308,7 +308,7 @@ class AristaL3DriverTestCasesMlagConfig(base.BaseTestCase):
                     'ip address %s' % router_ip,
                     'ip virtual-router address %s' % gw_ip, 'exit']
 
-            s.runCmds.assert_called_once_with(version=1, cmds=cmds)
+            s.execute.assert_called_once_with(cmds, keep_alive=True)
 
     def test_delete_interface_from_router_on_eos(self):
         router_name = 'test-router-1'
@@ -320,7 +320,7 @@ class AristaL3DriverTestCasesMlagConfig(base.BaseTestCase):
             cmds = ['enable', 'configure', 'no interface vlan %s' % segment_id,
                     'exit']
 
-            s.runCmds.assert_called_once_with(version=1, cmds=cmds)
+            s.execute.assert_called_once_with(cmds, keep_alive=True)
 
 
 class AristaL3DriverTestCases_v4(base.BaseTestCase):
@@ -491,8 +491,10 @@ class AristaL3DriverTestCasesMlag_one_switch_failed(base.BaseTestCase):
         tenant = '123'
 
         # Make one of the switches throw an exception - i.e. fail
-        self.drv._servers[0].runCmds = mock.Mock(side_effect=Exception)
-        self.drv.create_router(None, tenant, router)
+        self.drv._servers[0].execute = mock.Mock(side_effect=Exception)
+        with mock.patch.object(arista.LOG, 'exception') as log_exception:
+            self.drv.create_router(None, tenant, router)
+            log_exception.assert_called_once_with(mock.ANY)
 
     def test_delete_router_when_one_switch_fails(self):
         router = {}
@@ -501,8 +503,10 @@ class AristaL3DriverTestCasesMlag_one_switch_failed(base.BaseTestCase):
         router_id = '345'
 
         # Make one of the switches throw an exception - i.e. fail
-        self.drv._servers[1].runCmds = mock.Mock(side_effect=Exception)
-        self.drv.delete_router(None, tenant, router_id, router)
+        self.drv._servers[1].execute = mock.Mock(side_effect=Exception)
+        with mock.patch.object(arista.LOG, 'exception') as log_exception:
+            self.drv.delete_router(None, tenant, router_id, router)
+            log_exception.assert_called_once_with(mock.ANY)
 
     def test_add_router_interface_when_one_switch_fails(self):
         router = {}
@@ -514,8 +518,10 @@ class AristaL3DriverTestCasesMlag_one_switch_failed(base.BaseTestCase):
         router['gip'] = '10.10.10.1'
 
         # Make one of the switches throw an exception - i.e. fail
-        self.drv._servers[1].runCmds = mock.Mock(side_effect=Exception)
-        self.drv.add_router_interface(None, router)
+        self.drv._servers[1].execute = mock.Mock(side_effect=Exception)
+        with mock.patch.object(arista.LOG, 'exception') as log_exception:
+            self.drv.add_router_interface(None, router)
+            log_exception.assert_called_once_with(mock.ANY)
 
     def test_remove_router_interface_when_one_switch_fails(self):
         router = {}
@@ -527,5 +533,7 @@ class AristaL3DriverTestCasesMlag_one_switch_failed(base.BaseTestCase):
         router['gip'] = '10.10.10.1'
 
         # Make one of the switches throw an exception - i.e. fail
-        self.drv._servers[0].runCmds = mock.Mock(side_effect=Exception)
-        self.drv.remove_router_interface(None, router)
+        self.drv._servers[0].execute = mock.Mock(side_effect=Exception)
+        with mock.patch.object(arista.LOG, 'exception') as log_exception:
+            self.drv.remove_router_interface(None, router)
+            log_exception.assert_called_once_with(mock.ANY)
