@@ -16,9 +16,12 @@
 import mock
 from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants as n_const
+from neutron_lib.plugins.ml2 import api as driver_api
 
-from neutron.common import constants as neutron_const
-from neutron.plugins.ml2 import driver_api as api
+from neutron.db import models_v2
+from neutron.plugins.ml2 import models as port_models
+from neutron.services.trunk import callbacks
+from neutron.services.trunk import models as trunk_models
 from neutron.tests.unit import testlib_api
 
 from networking_arista.ml2 import mechanism_arista
@@ -42,7 +45,6 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
     def tearDown(self):
         super(AristaDriverTestCase, self).tearDown()
-        self.drv.stop_synchronization_thread()
 
     def test_create_network_precommit(self):
         tenant_id = 'ten-1'
@@ -527,7 +529,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
             mock.call.unplug_port_from_network(device_id, 'compute', host_id,
                                                port_id, network_id, tenant_id,
                                                None, vnic_type,
-                                               switch_bindings=profile),
+                                               switch_bindings=profile,
+                                               trunk_details=None),
             mock.call.remove_security_group(None, profile),
             mock.call.num_nets_provisioned(tenant_id),
             mock.call.num_vms_provisioned(tenant_id),
@@ -574,7 +577,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                                port_id, network_id,
                                                INTERNAL_TENANT_ID, None,
                                                vnic_type,
-                                               switch_bindings=profile),
+                                               switch_bindings=profile,
+                                               trunk_details=None),
             mock.call.remove_security_group(None, profile),
             mock.call.num_nets_provisioned(INTERNAL_TENANT_ID),
             mock.call.num_vms_provisioned(INTERNAL_TENANT_ID),
@@ -589,6 +593,196 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
 
+    def test_delete_trunk_port_postcommit(self):
+        # trunk port
+        tenant_id = 'ten-3'
+        network_id = 'net3-id'
+        segmentation_id = 1003
+        vm_id = 'vm3'
+
+        trunk_details = {'sub_ports': [{'mac_address': 'mac_address',
+                                        'port_id': 'subport_id',
+                                        'segmentation_id': 123,
+                                        'segmentation_type': 'vlan'}],
+                         'trunk_id': 'trunk_id'}
+
+        network_context = self._get_network_context(tenant_id,
+                                                    network_id,
+                                                    segmentation_id,
+                                                    False)
+        port_context = self._get_port_context(tenant_id,
+                                              network_id,
+                                              vm_id,
+                                              network_context)
+
+        mechanism_arista.db_lib.num_nets_provisioned.return_value = 0
+        mechanism_arista.db_lib.num_vms_provisioned.return_value = 0
+        mechanism_arista.db_lib.is_network_provisioned.return_value = True
+
+        port = port_context.current
+        port['trunk_details'] = trunk_details
+        device_id = port['device_id']
+        host_id = port['binding:host_id']
+        port_id = port['id']
+        vnic_type = port['binding:vnic_type']
+        profile = port['binding:profile']
+
+        network = {'tenant_id': tenant_id}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
+        physnet = dict(physnet='default')
+        self.fake_rpc.get_physical_network.return_value = physnet
+        self.drv.rpc.hpb_supported.return_value = True
+
+        self.drv.delete_port_postcommit(port_context)
+
+        expected_calls = [
+            mock.call.NeutronNets(),
+            mock.call.unplug_port_from_network(device_id, 'compute', host_id,
+                                               port_id, network_id, tenant_id,
+                                               None, vnic_type,
+                                               switch_bindings=profile,
+                                               trunk_details=trunk_details),
+            mock.call.remove_security_group(None, profile),
+            mock.call.num_nets_provisioned(tenant_id),
+            mock.call.num_vms_provisioned(tenant_id),
+            mock.call.forget_tenant(tenant_id),
+            mock.call.delete_tenant(tenant_id),
+            mock.call.hpb_supported(),
+        ]
+        for binding_level in port_context.binding_levels:
+            expected_calls.append(mock.call.is_network_provisioned(
+                tenant_id, network_id, None,
+                binding_level['bound_segment']['id']))
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
+    def test_set_subport(self):
+        tenant_id = 'ten-4'
+        network_id = 'net4-id'
+        vm_id = 'vm4'
+        trunk_id = 111
+
+        host_id = 'host1'
+        port_id = 'p1'
+        vnic_type = 'allowed'
+        profile = []
+        sg = ['security-groups']
+        orig_sg = None
+
+        trunk_port = models_v2.Port(
+            tenant_id=tenant_id, network_id='net-trunk',
+            device_id=vm_id, device_owner='compute:None')
+        trunk_port.port_binding = port_models.PortBinding()
+        trunk_port.port_binding.vnic_type = vnic_type
+        trunk_port.port_binding.host = host_id
+        trunk_port.port_binding.profile = profile
+
+        mechanism_arista.db_lib.num_nets_provisioned.return_value = 0
+        mechanism_arista.db_lib.num_vms_provisioned.return_value = 0
+        mechanism_arista.db_lib.get_trunk_port_by_trunk_id.return_value = \
+            trunk_port
+        mechanism_arista.db_lib.is_network_provisioned.return_value = True
+
+        resource = 'SubPort'
+        event = 'AFTER_CREATE'
+        trigger = 'AristaDriver'
+
+        sp = dict(models_v2.Port(
+            id=port_id,
+            device_owner='trunk:subport',
+            network_id=network_id,
+            name='subport'))
+        sp['security_groups'] = ['security-groups']
+        subport = trunk_models.SubPort()
+        subport.port_id = port_id
+        payload = callbacks.TrunkPayload(None, trunk_id,
+                                         subports=[subport])
+        segments = [{'segmentation_id': 12,
+                     'physical_network': 'default',
+                     'id': 'segment_id',
+                     'network_type': 'vlan'}]
+        bindings = []
+
+        self.drv.ndb.get_port.return_value = sp
+        self.drv.ndb.get_network_id_from_port_id.return_value = network_id
+        mechanism_arista.db_lib.get_network_segments_by_port_id.return_value = \
+            segments
+
+        self.drv.set_subport(resource, event, trigger, payload=payload)
+
+        expected_calls = [
+            mock.call.NeutronNets(),
+            mock.call.get_trunk_port_by_trunk_id(trunk_id),
+            mock.call.get_network_segments_by_port_id('p1'),
+            mock.call.plug_port_into_network(vm_id, host_id, port_id,
+                                             network_id, tenant_id,
+                                             'subport', 'trunk:subport',
+                                             sg, orig_sg, vnic_type,
+                                             segments=segments,
+                                             switch_bindings=bindings),
+        ]
+
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
+    def test_unset_subport(self):
+        tenant_id = 'ten-4'
+        network_id = 'net4-id'
+        vm_id = 'vm4'
+        trunk_id = 111
+
+        device_owner = 'trunk:subport'
+        host_id = 'host1'
+        port_id = 'p1'
+        vnic_type = 'allowed'
+        profile = []
+
+        trunk_port = models_v2.Port(
+            tenant_id=tenant_id, network_id=network_id,
+            device_id=vm_id, device_owner='compute:None')
+        trunk_port.port_binding = port_models.PortBinding()
+        trunk_port.port_binding.vnic_type = vnic_type
+        trunk_port.port_binding.host = host_id
+        trunk_port.port_binding.profile = profile
+
+        mechanism_arista.db_lib.num_nets_provisioned.return_value = 0
+        mechanism_arista.db_lib.num_vms_provisioned.return_value = 0
+        mechanism_arista.db_lib.get_trunk_port_by_trunk_id.return_value = \
+            trunk_port
+        mechanism_arista.db_lib.is_network_provisioned.return_value = True
+
+        resource = 'SubPort'
+        event = 'AFTER_DELETE'
+        trigger = 'AristaDriver'
+        subport = trunk_models.SubPort()
+        subport.port_id = port_id
+        payload = callbacks.TrunkPayload(None, trunk_id,
+                                         subports=[trunk_models.SubPort()])
+        sp = models_v2.Port(
+            id=port_id,
+            device_owner='trunk:subport',
+            network_id=network_id
+        )
+
+        self.drv.ndb.get_port.return_value = sp
+
+        self.drv.unset_subport(resource, event, trigger, payload=payload)
+
+        expected_calls = [
+            mock.call.NeutronNets(),
+            mock.call.get_trunk_port_by_trunk_id(trunk_id),
+            mock.call.unplug_port_from_network(vm_id, device_owner,
+                                               host_id, port_id, network_id,
+                                               tenant_id, [], vnic_type,
+                                               switch_bindings=profile,
+                                               trunk_details=None),
+            mock.call.remove_security_group([], profile),
+            mock.call.num_nets_provisioned(tenant_id),
+            mock.call.num_vms_provisioned(tenant_id),
+            mock.call.forget_tenant(tenant_id),
+            mock.call.delete_tenant(tenant_id),
+        ]
+
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
     def test_update_port_precommit(self):
         # Test the case where the port was not provisioned previsouly
         # If port is not provisioned, we should bail out
@@ -816,7 +1010,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                                orig_host_id, port_id,
                                                network_id, tenant_id,
                                                sg, vnic_type,
-                                               switch_bindings=profile),
+                                               switch_bindings=profile,
+                                               trunk_details=None),
             mock.call.remove_security_group(None, []),
             mock.call.num_nets_provisioned(tenant_id),
             mock.call.num_vms_provisioned(tenant_id),
@@ -848,6 +1043,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         mechanism_arista.db_lib.num_nets_provisioned.return_value = 1
         mechanism_arista.db_lib.num_vms_provisioned.return_value = 1
         self.drv.ndb.get_all_network_segments.return_value = segments
+        mechanism_arista.db_lib.hpb_supported.return_value = True
 
         network = {'tenant_id': tenant_id}
         self.drv.ndb.get_network_from_net_id.return_value = [network]
@@ -881,7 +1077,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                              port_name, device_owner, None,
                                              None, vnic_type,
                                              segments=segments,
-                                             switch_bindings=profile)
+                                             switch_bindings=profile,
+                                             trunk_details=None)
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -940,7 +1137,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                              port_name, device_owner, None,
                                              None, vnic_type,
                                              segments=segments,
-                                             switch_bindings=profile)
+                                             switch_bindings=profile,
+                                             trunk_details=None)
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -1000,7 +1198,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                              port_name, device_owner, None,
                                              None, vnic_type,
                                              segments=segments,
-                                             switch_bindings=profile)
+                                             switch_bindings=profile,
+                                             trunk_details=None)
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -1025,7 +1224,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                                port_id, network_id,
                                                tenant_id,
                                                None, vnic_type,
-                                               switch_bindings=profile),
+                                               switch_bindings=profile,
+                                               trunk_details=None),
             mock.call.remove_security_group(None, profile),
             mock.call.num_nets_provisioned(tenant_id),
             mock.call.num_vms_provisioned(tenant_id),
@@ -1033,6 +1233,74 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
 
+    def test_update_trunk_port_postcommit(self):
+        # trunk port
+        tenant_id = 'ten-4'
+        network_id = 'net4-id'
+        segmentation_id = 1004
+        vm_id = 'vm4'
+
+        trunk_details = {'sub_ports': [{'mac_address': 'mac_address',
+                                        'port_id': 'subport_id',
+                                        'segmentation_id': 123,
+                                        'segmentation_type': 'vlan'}],
+                         'trunk_id': 'trunk_id'}
+
+        network_context = self._get_network_context(tenant_id,
+                                                    network_id,
+                                                    segmentation_id,
+                                                    False)
+        segments = network_context.network_segments
+        port_context = self._get_port_context(tenant_id,
+                                              network_id,
+                                              vm_id,
+                                              network_context)
+
+        mechanism_arista.db_lib.is_port_provisioned.return_value = True
+        mechanism_arista.db_lib.is_network_provisioned.return_value = True
+        mechanism_arista.db_lib.get_shared_network_owner_id.return_value = 1
+        mechanism_arista.db_lib.num_nets_provisioned.return_value = 1
+        mechanism_arista.db_lib.num_vms_provisioned.return_value = 1
+        self.drv.ndb.get_all_network_segments.return_value = segments
+        mechanism_arista.db_lib.hpb_supported.return_value = True
+
+        network = {'tenant_id': tenant_id}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
+        port = port_context.current
+        port['trunk_details'] = trunk_details
+        device_id = port['device_id']
+        device_owner = port['device_owner']
+        host_id = port['binding:host_id']
+        port_id = port['id']
+        port_name = port['name']
+        vnic_type = port['binding:vnic_type']
+        profile = port['binding:profile']
+        network_name = network_context.current['name']
+
+        self.drv.update_port_postcommit(port_context)
+
+        expected_calls = [
+            mock.call.NeutronNets(),
+            mock.call.is_port_provisioned(port_id, None),
+            mock.call.is_network_provisioned(tenant_id, network_id,
+                                             None, None),
+            mock.call.is_network_provisioned(tenant_id, network_id,
+                                             segmentation_id, None),
+            mock.call.hpb_supported(),
+            mock.call.create_network_segments(tenant_id, network_id,
+                                              network_name,
+                                              segments),
+            mock.call.plug_port_into_network(device_id, host_id, port_id,
+                                             network_id, tenant_id,
+                                             port_name, device_owner, None,
+                                             None, vnic_type,
+                                             segments=segments,
+                                             switch_bindings=profile,
+                                             trunk_details=trunk_details)
+        ]
+
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
     def test_update_port_precommit_dhcp_reserved_port(self):
         '''Test to ensure the dhcp port migration is handled correctly.
 
@@ -1064,7 +1332,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         network_id = 'n1'
         old_device_id = 'old_device_id'
         new_device_id = 'new_device_id'
-        reserved_device = neutron_const.DEVICE_ID_RESERVED_DHCP_PORT
+        reserved_device = n_const.DEVICE_ID_RESERVED_DHCP_PORT
         old_host = 'ubuntu1'
         new_host = 'ubuntu2'
         port_id = 101
@@ -1213,7 +1481,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         network_id = 'n1'
         old_device_id = 'old_device_id'
         new_device_id = 'new_device_id'
-        reserved_device = neutron_const.DEVICE_ID_RESERVED_DHCP_PORT
+        reserved_device = n_const.DEVICE_ID_RESERVED_DHCP_PORT
         old_host = 'ubuntu1'
         new_host = 'ubuntu2'
         port_id = 101
@@ -1263,7 +1531,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                                port_id, network_id,
                                                tenant_id,
                                                None, vnic_type,
-                                               switch_bindings=profile),
+                                               switch_bindings=profile,
+                                               trunk_details=None),
             mock.call.remove_security_group(None, profile),
             mock.call.num_nets_provisioned(tenant_id),
             mock.call.num_vms_provisioned(tenant_id),
@@ -1294,7 +1563,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                                port_id, network_id,
                                                tenant_id,
                                                None, vnic_type,
-                                               switch_bindings=profile),
+                                               switch_bindings=profile,
+                                               trunk_details=None),
             mock.call.remove_security_group(None, profile),
             mock.call.num_nets_provisioned(tenant_id),
             mock.call.num_vms_provisioned(tenant_id),
@@ -1332,7 +1602,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                              n_const.DEVICE_OWNER_DHCP,
                                              None, None, vnic_type,
                                              segments=[],
-                                             switch_bindings=profile),
+                                             switch_bindings=profile,
+                                             trunk_details=None),
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -1364,7 +1635,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                              n_const.DEVICE_OWNER_DHCP,
                                              None, None, vnic_type,
                                              segments=[],
-                                             switch_bindings=profile),
+                                             switch_bindings=profile,
+                                             trunk_details=None),
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -1492,6 +1764,21 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         return FakePortContext(port, dict(orig_port), network, None,
                                binding_levels)
 
+    def test_supported_device_owner(self):
+        device_owner_list = [n_const.DEVICE_OWNER_DHCP,
+                             n_const.DEVICE_OWNER_DVR_INTERFACE,
+                             'compute:*',
+                             'baremetal:*',
+                             'trunk:*']
+        for device_owner in device_owner_list:
+            self.assertTrue(self.drv._supported_device_owner(device_owner),
+                            'device_owner:%s should be a '
+                            'supported device owner' % device_owner)
+        device_owner = 'compute:probe'
+        self.assertFalse(self.drv._supported_device_owner(device_owner),
+                         'device_owner:%s is not a '
+                         'supported device owner' % device_owner)
+
 
 class fake_keystone_info_class(object):
     """To generate fake Keystone Authentication token information
@@ -1499,7 +1786,7 @@ class fake_keystone_info_class(object):
     Arista Driver expects Keystone auth info. This fake information
     is for testing only
     """
-    auth_uri = 'abc://host:35357/v2.0/'
+    auth_uri = 'abc://host:35357/v3/'
     identity_uri = 'abc://host:5000'
     admin_user = 'neutron'
     admin_password = 'fun'
@@ -1575,16 +1862,18 @@ class FakePortContext(object):
     def binding_levels(self):
         if self._binding_levels:
             return [{
-                api.BOUND_DRIVER: level.driver,
-                api.BOUND_SEGMENT: self._expand_segment(level.segment_id)
+                driver_api.BOUND_DRIVER: level.driver,
+                driver_api.BOUND_SEGMENT:
+                    self._expand_segment(level.segment_id)
             } for level in self._binding_levels]
 
     @property
     def original_binding_levels(self):
         if self._original_binding_levels:
             return [{
-                api.BOUND_DRIVER: level.driver,
-                api.BOUND_SEGMENT: self._expand_segment(level.segment_id)
+                driver_api.BOUND_DRIVER: level.driver,
+                driver_api.BOUND_SEGMENT: (
+                    self._expand_segment(level.segment_id))
             } for level in self._original_binding_levels]
 
     @property
@@ -1594,7 +1883,7 @@ class FakePortContext(object):
 
     def _expand_segment(self, segment_id):
         for segment in self._network_context.network_segments:
-            if segment[api.ID] == segment_id:
+            if segment[driver_api.ID] == segment_id:
                 return segment
 
     def release_dynamic_segment(self, segment_id):
